@@ -1,7 +1,7 @@
 #
 # InspIRCd -- Internet Relay Chat Daemon
 #
-#   Copyright (C) 2016-2022 Sadie Powell <sadie@witchery.services>
+#   Copyright (C) 2016-2024 Sadie Powell <sadie@witchery.services>
 #
 # This file is part of InspIRCd.  InspIRCd is free software: you can
 # redistribute it and/or modify it under the terms of the GNU General Public
@@ -19,7 +19,7 @@
 
 package make::directive;
 
-use v5.10.0;
+use v5.26.0;
 use strict;
 use warnings FATAL => qw(all);
 
@@ -27,31 +27,41 @@ use File::Basename        qw(dirname);
 use File::Spec::Functions qw(catdir);
 use Exporter              qw(import);
 
+use make::common;
 use make::configure;
 use make::console;
 
 use constant DIRECTIVE_ERROR_PIPE => $ENV{INSPIRCD_VERBOSE} ? '' : '2>/dev/null';
 use constant PKG_CONFIG           => $ENV{PKG_CONFIG} || 'pkg-config';
-use constant VENDOR_DIRECTORY     => catdir(dirname(dirname(__FILE__)), 'vendor');
 
-our @EXPORT = qw(get_directive
-                 execute_functions);
+our @EXPORT = qw(
+	get_directives
+	get_directive
+	execute_functions
+);
 
-sub get_directive($$;$)
-{
-	my ($file, $property, $default) = @_;
-	open(my $fh, $file) or return $default;
+sub get_directives($$;$) {
+	my ($file, $property, $functions) = @_;
+	$functions //= 1;
+	open(my $fh, $file) or return ();
 
-	my $value = '';
+	my @values;
 	while (<$fh>) {
 		if ($_ =~ /^\/\* \$(\S+): (.+) \*\/$/ || $_ =~ /^\/\/\/ \$(\S+): (.+)/) {
 			next unless $1 eq $property;
-			$value .= ' ' . execute_functions($file, $1, $2);
+			my $value = $functions ? execute_functions($file, $1, $2) : $2;
+			push @values, $value;
 		}
 	}
 	close $fh;
+	return @values;
+}
 
-	# Strip all extraneous whitespace.
+sub get_directive($$;$$) {
+	my ($file, $property, $default, $functions) = @_;
+	my @values = get_directives($file, $property, $functions);
+
+	my $value = join ' ', @values;
 	$value =~ s/^\s+|\s+$//g;
 	return $value || $default;
 }
@@ -62,7 +72,7 @@ sub execute_functions($$$) {
 	# NOTE: we have to use 'our' instead of 'my' here because of a Perl bug.
 	for (our @parameters = (); $line =~ /([a-z_]+)\((?:\s*"([^"]*)(?{push @parameters, $2})"\s*)*\)/; undef @parameters) {
 		my $sub = make::directive->can("__function_$1");
-		print_error "unknown $name directive '$1' in $file!" unless $sub;
+		print_error "unknown $name function '$1' in $file!" unless $sub;
 
 		# Call the subroutine and replace the function.
 		my $result = $sub->($file, @parameters);
@@ -126,17 +136,15 @@ sub __error {
 	# If we have author information then tell the user to report the bug
 	# to them. Otherwise, assume it is a bundled module and tell the user
 	# to report it to the InspIRCd issue tracker.
-	my $author = get_directive($file, 'ModAuthor');
-	if (defined $author) {
+	my @authors = get_directives($file, 'ModAuthor');
+	if (@authors) {
 		push @message, 'If you believe this error to be a bug then you can try to contact the';
-		push @message, 'author of this module:';
-		my $author_mail = get_directive($file, 'ModAuthorMail');
-		if (defined $author_mail) {
-			push @message, " * $author <$author_mail>";
-		} else {
+		push @message, "author${$#authors ? \'s' : \''} of this module:";
+		for my $author (@authors) {
 			push @message, " * $author";
 		}
 	} else {
+		my %version = get_version();
 		push @message, 'If you believe this error to be a bug then you can file a bug report';
 		push @message, 'at https://github.com/inspircd/inspircd/issues. Please include the';
 		push @message, 'following data in your report:';
@@ -148,7 +156,7 @@ sub __error {
 		push @message, " * PKG_CONFIG_PATH: ${\__env_or_unset 'PKG_CONFIG_PATH'}";
 		push @message, '';
 		push @message, 'You can also refer to the documentation page for this module at';
-		push @message, "https://docs.inspircd.org/3/modules/${\module_shrink $file}.";
+		push @message, "https://docs.inspircd.org/$version{MAJOR}/modules/${\module_shrink $file}.";
 	}
 	push @message, '';
 
@@ -254,80 +262,87 @@ sub __function_find_linker_flags {
 sub __function_require_compiler {
 	my ($file, $name, $minimum, $maximum) =  @_;
 
+	# Check for an inverted match.
+	my ($ok, $err) = ("", undef);
+	if ($name =~ s/^!//) {
+		($ok, $err) = ($err, $ok);
+	}
+
 	# Look up information about the compiler.
-	return undef unless $ENV{CXX};
+	return $err unless $ENV{CXX};
 	my %compiler = get_compiler_info($ENV{CXX});
 
 	# Check whether the current compiler is suitable.
-	return undef unless $compiler{NAME} eq $name;
-	return undef if defined $minimum && $compiler{VERSION} < $minimum;
-	return undef if defined $maximum && $compiler{VERSION} > $maximum;
+	return $err unless $compiler{NAME} eq $name;
+	return $err if defined $minimum && $compiler{VERSION} < $minimum;
+	return $err if defined $maximum && $compiler{VERSION} > $maximum;
 
 	# Requirement directives don't change anything directly.
-	return "";
+	return $ok;
 }
 
 sub __function_require_system {
 	my ($file, $name, $minimum, $maximum) = @_;
-	my ($system, $version);
+	my ($system, $system_like, $version);
+
+	# Check for an inverted match.
+	my ($ok, $err) = ("", undef);
+	if ($name =~ s/^!//) {
+		($ok, $err) = ($err, $ok);
+	}
+
+	# If a system name ends in a tilde we match on alternate names.
+	my $match_like = $name =~ s/~$//;
 
 	# Linux is special and can be compared by distribution names.
 	if ($^O eq 'linux' && $name ne 'linux') {
-		chomp($system = lc `lsb_release --id --short 2>/dev/null`);
-		chomp($version = lc `lsb_release --release --short 2>/dev/null`);
+		chomp($system      = lc `sh -c '. /etc/os-release 2>/dev/null && echo \$ID'`);
+		chomp($system_like = lc `sh -c '. /etc/os-release 2>/dev/null && echo \$ID_LIKE'`);
+		chomp($version     = lc `sh -c '. /etc/os-release 2>/dev/null && echo \$VERSION_ID'`);
 	}
 
 	# Gather information on the system if we don't have it already.
 	chomp($system ||= lc `uname -s 2>/dev/null`);
 	chomp($version ||= lc `uname -r 2>/dev/null`);
+	$system_like ||= '';
 
 	# We only care about the important bit of the version number so trim the rest.
 	$version =~ s/^(\d+\.\d+).+/$1/;
 
 	# Check whether the current system is suitable.
-	return undef if $name ne $system;
-	return undef if defined $minimum && $version < $minimum;
-	return undef if defined $maximum && $version > $maximum;
+	if ($name ne $system) {
+		return $err unless $match_like;
+		return $err unless grep { $_ eq $name } split /\s+/, $system_like;
+	}
+	return $err if defined $minimum && $version < $minimum;
+	return $err if defined $maximum && $version > $maximum;
 
 	# Requirement directives don't change anything directly.
-	return "";
+	return $ok;
 }
 
-sub __function_require_version {
+sub __function_require_library {
 	my ($file, $name, $minimum, $maximum) = @_;
 
+	# Check for an inverted match.
+	my ($ok, $err) = ("", undef);
+	if ($name =~ s/^!//) {
+		($ok, $err) = ($err, $ok);
+	}
+
 	# If pkg-config isn't installed then we can't do anything here.
-	if (system "pkg-config --exists $name ${\DIRECTIVE_ERROR_PIPE}") {
+	if (system "${\PKG_CONFIG} --version 1>/dev/null 2>/dev/null") {
 		print_warning "unable to look up the version of <|GREEN $name|> using pkg-config!";
-		return undef;
+		return $err;
 	}
 
 	# Check with pkg-config whether we have the required version.
-	return undef if defined $minimum && system "pkg-config --atleast-version $minimum $name";
-	return undef if defined $maximum && system "pkg-config --max-version $maximum $name";
+	return $err if system "${\PKG_CONFIG} --exists $name";
+	return $err if defined $minimum && system "${\PKG_CONFIG} --atleast-version $minimum $name";
+	return $err if defined $maximum && system "${\PKG_CONFIG} --max-version $maximum $name";
 
 	# Requirement directives don't change anything directly.
-	return "";
-}
-
-sub __function_vendor_directory {
-	my ($file, $name) = @_;
-
-	# Try to look the directory up in the environment...
-	my $key = __environment 'INSPIRCD_VENDOR_', $name;
-	if (defined $ENV{$key}) {
-		say console_format "Found the <|GREEN $name|> vendor directory for <|GREEN ${\module_shrink $file}|> using the environment: <|BOLD $ENV{$key}|>";
-		return $ENV{$key};
-	}
-
-	my $directory = catdir(VENDOR_DIRECTORY, $name);
-	if (-d $directory) {
-		say console_format "Using the default <|GREEN $name|> vendor directory for <|GREEN ${\module_shrink $file}|>: <|BOLD $directory|>";
-		return $directory;
-	}
-
-	# We can't find it via the environment or via the filesystem so give up.
-	__error $file, "unable to find the <|GREEN $name|> vendor directory for <|GREEN ${\module_shrink $file}|>!";
+	return $ok;
 }
 
 sub __function_warning {
