@@ -1,14 +1,14 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
- *   Copyright (C) 2018, 2020, 2022 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2018, 2020-2024 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2018 linuxdaemon <linuxdaemon.irc@gmail.com>
  *   Copyright (C) 2017 B00mX0r <b00mx0r@aureus.pw>
  *   Copyright (C) 2013-2016, 2018 Attila Molnar <attilamolnar@hush.com>
- *   Copyright (C) 2012, 2019 Robby <robby@chatbelgie.be>
+ *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2009-2010 Daniel De Graaf <danieldg@inspircd.org>
- *   Copyright (C) 2008 Thomas Stagner <aquanight@inspircd.org>
- *   Copyright (C) 2007-2008 Robin Burchell <robin+git@viroteck.net>
+ *   Copyright (C) 2008 Thomas Stagner <aquanight@gmail.com>
+ *   Copyright (C) 2008 Robin Burchell <robin+git@viroteck.net>
  *   Copyright (C) 2007 Dennis Friis <peavey@inspircd.org>
  *   Copyright (C) 2005-2008 Craig Edwards <brain@inspircd.org>
  *
@@ -27,49 +27,55 @@
 
 
 #include "inspircd.h"
+#include "clientprotocolmsg.h"
+#include "numerichelper.h"
+#include "timeutils.h"
+
 #include "core_channel.h"
 #include "invite.h"
 
 enum
 {
+	// From RFC 1459.
+	RPL_INVITING = 341,
+	ERR_USERONCHANNEL = 443,
+
 	// From ircd-hybrid.
 	RPL_INVITELIST = 336,
 	RPL_ENDOFINVITELIST = 337
 };
 
 CommandInvite::CommandInvite(Module* parent, Invite::APIImpl& invapiimpl)
-	: Command(parent, "INVITE", 0, 0)
+	: Command(parent, "INVITE")
 	, invapi(invapiimpl)
 {
-	Penalty = 4;
-	syntax = "[<nick> <channel> [<time>]]";
+	penalty = 4000;
+	syntax = { "[<nick> <channel> [<time>]]" };
 }
 
-/** Handle /INVITE
- */
 CmdResult CommandInvite::Handle(User* user, const Params& parameters)
 {
-	ModResult MOD_RESULT;
+	ModResult modres;
 
 	if (parameters.size() >= 2)
 	{
 		User* u;
 		if (IS_LOCAL(user))
-			u = ServerInstance->FindNickOnly(parameters[0]);
+			u = ServerInstance->Users.FindNick(parameters[0], true);
 		else
-			u = ServerInstance->FindNick(parameters[0]);
+			u = ServerInstance->Users.Find(parameters[0], true);
 
-		Channel* c = ServerInstance->FindChan(parameters[1]);
+		auto* c = ServerInstance->Channels.Find(parameters[1]);
 		time_t timeout = 0;
 		if (parameters.size() >= 3)
 		{
 			if (IS_LOCAL(user))
 			{
 				unsigned long duration;
-				if (!InspIRCd::Duration(parameters[2], duration))
+				if (!Duration::TryFrom(parameters[2], duration))
 				{
 					user->WriteNotice("*** Invalid duration for invite");
-					return CMD_FAILURE;
+					return CmdResult::FAILURE;
 				}
 				timeout = ServerInstance->Time() + duration;
 			}
@@ -80,12 +86,12 @@ CmdResult CommandInvite::Handle(User* user, const Params& parameters)
 		if (!c)
 		{
 			user->WriteNumeric(Numerics::NoSuchChannel(parameters[1]));
-			return CMD_FAILURE;
+			return CmdResult::FAILURE;
 		}
-		if ((!u) || (u->registered != REG_ALL))
+		if (!u)
 		{
 			user->WriteNumeric(Numerics::NoSuchNick(parameters[0]));
-			return CMD_FAILURE;
+			return CmdResult::FAILURE;
 		}
 
 		// Verify channel timestamp if the INVITE is coming from a remote server
@@ -93,41 +99,41 @@ CmdResult CommandInvite::Handle(User* user, const Params& parameters)
 		{
 			// Remote INVITE commands must carry a channel timestamp
 			if (parameters.size() < 3)
-				return CMD_INVALID;
+				return CmdResult::INVALID;
 
 			// Drop the invite if our channel TS is lower
 			time_t RemoteTS = ConvToNum<time_t>(parameters[2]);
 			if (c->age < RemoteTS)
-				return CMD_FAILURE;
+				return CmdResult::FAILURE;
 		}
 
 		if ((IS_LOCAL(user)) && (!c->HasUser(user)))
 		{
 			user->WriteNumeric(ERR_NOTONCHANNEL, c->name, "You're not on that channel!");
-			return CMD_FAILURE;
+			return CmdResult::FAILURE;
 		}
 
 		if (c->HasUser(u))
 		{
 			user->WriteNumeric(ERR_USERONCHANNEL, u->nick, c->name, "is already on channel");
-			return CMD_FAILURE;
+			return CmdResult::FAILURE;
 		}
 
-		FIRST_MOD_RESULT(OnUserPreInvite, MOD_RESULT, (user,u,c,timeout));
+		FIRST_MOD_RESULT(OnUserPreInvite, modres, (user, u, c, timeout));
 
-		if (MOD_RESULT == MOD_RES_DENY)
+		if (modres == MOD_RES_DENY)
 		{
-			return CMD_FAILURE;
+			return CmdResult::FAILURE;
 		}
-		else if (MOD_RESULT == MOD_RES_PASSTHRU)
+		else if (modres == MOD_RES_PASSTHRU)
 		{
 			if (IS_LOCAL(user))
 			{
-				unsigned int rank = c->GetPrefixValue(user);
+				ModeHandler::Rank rank = c->GetPrefixValue(user);
 				if (rank < HALFOP_VALUE)
 				{
 					user->WriteNumeric(Numerics::ChannelPrivilegesNeeded(c, HALFOP_VALUE, "send an invite"));
-					return CMD_FAILURE;
+					return CmdResult::FAILURE;
 				}
 			}
 		}
@@ -144,11 +150,11 @@ CmdResult CommandInvite::Handle(User* user, const Params& parameters)
 		{
 			user->WriteNumeric(RPL_INVITING, u->nick, c->name);
 			if (u->IsAway())
-				user->WriteNumeric(RPL_AWAY, u->nick, u->awaymsg);
+				user->WriteNumeric(RPL_AWAY, u->nick, u->away->message);
 		}
 
 		char prefix = 0;
-		unsigned int minrank = 0;
+		ModeHandler::Rank minrank = 0;
 		switch (invapi.announceinvites)
 		{
 			case Invite::ANNOUNCE_OPS:
@@ -184,7 +190,7 @@ CmdResult CommandInvite::Handle(User* user, const Params& parameters)
 		if (invapi.announceinvites != Invite::ANNOUNCE_NONE)
 		{
 			excepts.insert(user);
-			ClientProtocol::Messages::Privmsg privmsg(ServerInstance->FakeClient, c, InspIRCd::Format("*** %s invited %s into the channel", user->nick.c_str(), u->nick.c_str()), MSG_NOTICE);
+			ClientProtocol::Messages::Privmsg privmsg(ServerInstance->FakeClient, c, INSP_FORMAT("*** {} invited {} into the channel", user->nick, u->nick), MessageType::NOTICE);
 			c->Write(ServerInstance->GetRFCEvents().privmsg, privmsg, prefix, excepts);
 		}
 	}
@@ -195,12 +201,12 @@ CmdResult CommandInvite::Handle(User* user, const Params& parameters)
 		const Invite::List* list = invapi.GetList(IS_LOCAL(user));
 		if (list)
 		{
-			for (Invite::List::const_iterator i = list->begin(); i != list->end(); ++i)
-				user->WriteNumeric(RPL_INVITELIST, (*i)->chan->name);
+			for (const auto* invite : *list)
+				user->WriteNumeric(RPL_INVITELIST, invite->chan->name);
 		}
 		user->WriteNumeric(RPL_ENDOFINVITELIST, "End of INVITE list");
 	}
-	return CMD_SUCCESS;
+	return CmdResult::SUCCESS;
 }
 
 RouteDescriptor CommandInvite::GetRouting(User* user, const Params& parameters)
