@@ -1,11 +1,11 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
- *   Copyright (C) 2013, 2018 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2013, 2019-2021 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2012-2013 Attila Molnar <attilamolnar@hush.com>
- *   Copyright (C) 2012, 2019 Robby <robby@chatbelgie.be>
+ *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2009-2010 Daniel De Graaf <danieldg@inspircd.org>
- *   Copyright (C) 2009 Uli Schlachter <psychon@inspircd.org>
+ *   Copyright (C) 2009 Uli Schlachter <psychon@znc.in>
  *   Copyright (C) 2008 Craig Edwards <brain@inspircd.org>
  *
  * This file is part of InspIRCd.  InspIRCd is free software: you can
@@ -24,16 +24,24 @@
 
 #include "inspircd.h"
 
-class ModuleMapHide : public Module
+class ModuleMapHide final
+	: public Module
 {
+private:
 	std::string url;
- public:
-	void ReadConfig(ConfigStatus& status) CXX11_OVERRIDE
+
+public:
+	ModuleMapHide()
+		: Module(VF_VENDOR, "Allows the server administrator to replace the output of a /MAP and /LINKS with an URL.")
+	{
+	}
+
+	void ReadConfig(ConfigStatus& status) override
 	{
 		url = ServerInstance->Config->ConfValue("security")->getString("maphide");
 	}
 
-	ModResult OnPreCommand(std::string& command, CommandBase::Params& parameters, LocalUser* user, bool validated) CXX11_OVERRIDE
+	ModResult OnPreCommand(std::string& command, CommandBase::Params& parameters, LocalUser* user, bool validated) override
 	{
 		if (validated && !user->IsOper() && !url.empty() && (command == "MAP" || command == "LINKS"))
 		{
@@ -43,11 +51,6 @@ class ModuleMapHide : public Module
 		else
 			return MOD_RES_PASSTHRU;
 	}
-
-	Version GetVersion() CXX11_OVERRIDE
-	{
-		return Version("Allows the server administrator to replace the output of a /MAP and /LINKS with an URL.", VF_VENDOR);
-	}
 };
 
 MODULE_INIT(ModuleMapHide)
