@@ -1,10 +1,9 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
- *   Copyright (C) 2013, 2021 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2020-2021 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
- *   Copyright (C) 2012 Attila Molnar <attilamolnar@hush.com>
- *   Copyright (C) 2009 Uli Schlachter <psychon@inspircd.org>
+ *   Copyright (C) 2009 Uli Schlachter <psychon@znc.in>
  *   Copyright (C) 2009 Daniel De Graaf <danieldg@inspircd.org>
  *   Copyright (C) 2008 Robin Burchell <robin+git@viroteck.net>
  *
@@ -23,28 +22,34 @@
 
 
 #include "inspircd.h"
+#include "modules/extban.h"
 
-class ModuleServerBan : public Module
+class ServerExtBan final
+	: public ExtBan::MatchingBase
 {
- public:
-	Version GetVersion() CXX11_OVERRIDE
+public:
+	ServerExtBan(Module* Creator)
+		: ExtBan::MatchingBase(Creator, "server", 's')
 	{
-		return Version("Adds extended ban s: (server) which check whether users are on a server matching the specified glob pattern.", VF_OPTCOMMON|VF_VENDOR);
 	}
 
-	ModResult OnCheckBan(User *user, Channel *c, const std::string& mask) CXX11_OVERRIDE
+	bool IsMatch(User* user, Channel* channel, const std::string& text) override
 	{
-		if ((mask.length() > 2) && (mask[0] == 's') && (mask[1] == ':'))
-		{
-			if (InspIRCd::Match(user->server->GetPublicName(), mask.substr(2)))
-				return MOD_RES_DENY;
-		}
-		return MOD_RES_PASSTHRU;
+		return InspIRCd::Match(user->server->GetPublicName(), text);
 	}
+};
 
-	void On005Numeric(std::map<std::string, std::string>& tokens) CXX11_OVERRIDE
+class ModuleServerBan final
+	: public Module
+{
+private:
+	ServerExtBan extban;
+
+public:
+	ModuleServerBan()
+		: Module(VF_VENDOR | VF_OPTCOMMON, "Adds extended ban s: (server) which check whether users are on a server matching the specified glob pattern.")
+		, extban(this)
 	{
-		tokens["EXTBAN"].push_back('s');
 	}
 };
 
